// define constants for input & output pins
const int LedRed = 14;		// D5
const int LedGreen1 = 12;	// D6
const int LedGreen2 = 13;	// D7
const int SelectInput = 10;	// SD3 (D8 causes comms errors during upload?!)

// Initialize the OLED display using Wire library
SSD1306Wire  displayInfo(0x3c, D3, D4); // SDA=D3 SCL=D5, INFO display for weather info
SSD1306Wire  displayTemp(0x3c, D1, D2); // SDA=D1 SCL=D2, TEMP/RAIN display

// general constants
const int LoopTime = 3000; // in mSec 
const int IntervalDataFetch = 100; // interval internet host fetch = IntervalDataFetch * LoopTime = 3000 mSec x 100 = 300 Sec = 5 min

WiFiClient clientH;			// to connect to an insecure (HTTP) internet server
const int httpPort = 80;

WiFiClientSecure clientS;	// to connect to a secure (HTTPS) internet server
const int httpsPort = 443;

// constants for time server
const char* hostT = "rjut.nl";

// constants for weather server
const char* hostW = "weerlive.nl";
const String urlW = "/api/json-data-10min.php?key=" + String(PRIVATE_WEATHER_APIKEY) + "&locatie=" + String(PRIVATE_CITY);

// constants for rain server
const char* hostR = "gpsgadget.buienradar.nl";
const int httpsPortR = 443;
const String urlR = "/data/raintext?" + String(PRIVATE_COORDINATES);

// constants for solar server
const char* hostS = "monitoringapi.solaredge.com";

// variables for weather info:
bool WeatherInfoFound = 0;
String Humidity;			// eg 67 <%>
String TemperatureAir;		// eg 4.5 <degree>
String TemperatureFeel;		// eg 1.0 <degree>
String DagTempMin;			// min temp predicted for today
String DagTempMax;			// max temp predicted for today
String TempLow = "40.0";	// holds the lowest TemperatureAir value of today
String TempHigh = "-20.0";	// holds the highest TemperatureAir value of today 
String WindForce;			// eg 3 <Beaufort>
String WindDirection;		// eg N, NW, NNW, Noord
String AirPressure;			// eg 1023.45 <mBar>
int AirPressurePrev = 0;	// stores the last measure value of the previous interval
int AirPressureVal = 0;		// holds AirPressure as a value
String AirPressureTrend;	// eq --- -- - + ++ +++ indicating fall/rise of pressure since last interval
String Visibility;			// eg 14 <km>
String RainChance;			// eg 10 <%>
String SunChance;			// eg 80 <%>
String WeatherInfo;			// eg "Vrijwel onbewolkt"
String SunRiseTime;			// eg "06:46"
String SunSetTime;			// eg "21:24"

// variables for weather alarm info:
String AlarmText;
int AlarmCode;

// variables for rain info:
bool RainInfoFound;
int Rain5minData[24]; // array that holds the rain prediction of a 5 min interval

// variables for solar info:
bool SolarInfoFound = 0;
int SolarHourData[23]; // array that holds the measured solar energy of a particular hour of the day

// enumeration for modes button
enum DISPLAY_MODE {DISPLAY_MODE_WEATHER = 0,DISPLAY_MODE_RAIN = 1,DISPLAY_MODE_SOLAR = 2};

// define  global variables (start with uppercase letter, local variables start with lower case)
String CurrentDate = "2020-01-01";
String CurrentTime = "00:00";
String CurrentHour;			// eg "23" holds the hour digits of the current cycle
String PreviousHour;		// eg "23" holds the hour digits of the previous cycle
int DisplayMode = DISPLAY_MODE_WEATHER;
int InfoFunction = 0;
int FetchData = 0;
long CurrHeap = 0;
long PrevHeap = 0;
int ConnectsCount  = 0;
