void loop() {
	if (WiFi.status() != WL_CONNECTED) connectLocalWiFi();
	
	if (FetchData==0) {
		// this section is processed every few minutes. Delay = IntervalDataFetch x LoopTime 
		DPRINTLN("**************************************** START OF FETCH **********************************************************");
		getLocalDateTime();
		WeatherInfoFound = getWeatherInfo();
		RainInfoFound = getRainInfo();
		SolarInfoFound = getSolarInfo();			 
	}

	// this section is processed every few seconds.

	// ********************
	// *** INFO DISPLAY ***
	// ********************

	if (WeatherInfoFound) {
		// now write all found data to the INFO display buffer
		// first clear the display buffer
		displayInfo.clear();
		// On the 1st line of the display show City location
		// WeatherStation name:
		displayInfo.setFont(ArialMT_Plain_24);
		displayInfo.setTextAlignment(TEXT_ALIGN_LEFT);    
		displayInfo.drawString(0, 0, PRIVATE_CITY);
	
		// On the 2nd line of the display show wind direction and force and humidity
		// Wind direction+force:
		displayInfo.setFont(ArialMT_Plain_24);
		displayInfo.setTextAlignment(TEXT_ALIGN_LEFT);    
		displayInfo.drawString(0, 24, WindDirection + WindForce);
		// Humidity:
		displayInfo.setFont(ArialMT_Plain_24);
		displayInfo.setTextAlignment(TEXT_ALIGN_RIGHT);    
		displayInfo.drawString(128, 24, Humidity+ "%");
	
		// On the 3rd line of the display show alternating info
		displayInfo.setFont(ArialMT_Plain_16);
		displayInfo.setTextAlignment(TEXT_ALIGN_CENTER);    
		if (InfoFunction == 0) displayInfo.drawString(64, 48, CurrentDate + " " + CurrentTime);
		if (InfoFunction == 1) displayInfo.drawString(64, 48, TempLow + " < T < " + TempHigh);
		if (InfoFunction == 2) displayInfo.drawString(64, 48, AirPressure + " mBar "+ AirPressureTrend);
		if (InfoFunction == 3) displayInfo.drawString(64, 48, "Regen: "+ RainChance +" %");
		if (InfoFunction == 4) displayInfo.drawString(64, 48, "Zonkans: " + SunChance + " %");
		if (InfoFunction == 5) displayInfo.drawString(64, 48, "Zon: "+SunRiseTime + " - " + SunSetTime);
		if (InfoFunction == 6) displayInfo.drawString(64, 48, "Zicht: " + Visibility + " km");
		if (InfoFunction == 7) displayInfo.drawString(64, 48, String(ConnectsCount)+ " Alarm: = " + AlarmText);
		if (InfoFunction == 8) displayInfo.drawString(64, 48, WeatherInfo);  // may not fit on display line!!
	}
	else {
		displayInfo.setTextAlignment(TEXT_ALIGN_LEFT);    
		displayInfo.setFont(ArialMT_Plain_16);
		displayInfo.drawString(0, 24,"NO WEATHER INFO!");
	}
	
	// now actually show the data on the INFO display
	displayInfo.display();

	// *******************
	// *** MODE BUTTON ***
	// *******************

	// read the button selection input and step the mode if button is pressed
	if (digitalRead(SelectInput)==LOW) {
		switch (DisplayMode) {
  			case DISPLAY_MODE_WEATHER:
				DisplayMode = DISPLAY_MODE_RAIN;
				break;
			case DISPLAY_MODE_RAIN:
				DisplayMode = DISPLAY_MODE_SOLAR;
			    break;
			case DISPLAY_MODE_SOLAR:
				DisplayMode = DISPLAY_MODE_WEATHER;
			    break;
			default:
				DisplayMode = DISPLAY_MODE_WEATHER;
				break;
		}
	DPRINT("Display mode: ");DPRINTLN(DisplayMode);
	}

	// ********************
	// *** TEMP DISPLAY ***
	// ********************

	// write all found data to the TEMP display buffer
	displayTemp.clear();

	if (DisplayMode==DISPLAY_MODE_WEATHER) {
		if (WeatherInfoFound) { // only if data was found
			// Unit degrees symbol in small font flush right on the first line:
			displayTemp.setFont(ArialMT_Plain_16);
			displayTemp.setTextAlignment(TEXT_ALIGN_RIGHT);    
			displayTemp.drawString(128, 0,"c");
			// TemperatureAir in large font:
			displayTemp.setFont(Lato_Regular_52);
			// DPRINTLN("Breedte str Temp= "+ String(displayTemp.getStringWidth(TemperatureAir)));
			// temp string is max "-xx.x" In this font this is 0x13+0x1E+0x1E+0x0C+0x1E=0x79 => max 121 pixels wide
			displayTemp.drawString(121, -6, TemperatureAir);
			// TemperatureMin and Max in small font below current temperature
			displayTemp.setFont(ArialMT_Plain_16);
			displayTemp.setTextAlignment(TEXT_ALIGN_LEFT);    
			displayTemp.drawString(0, 48, DagTempMin);
			displayTemp.setTextAlignment(TEXT_ALIGN_CENTER);    
			displayTemp.drawString(64, 48, TemperatureFeel);
			displayTemp.setTextAlignment(TEXT_ALIGN_RIGHT);    
			displayTemp.drawString(128, 48, DagTempMax);
		}
		else { // display no info text
			displayTemp.setFont(ArialMT_Plain_16);
			displayTemp.setTextAlignment(TEXT_ALIGN_LEFT);    
			displayTemp.drawString(0, 24, "NO TEMP INFO!");
		}
	}
	if (DisplayMode==DISPLAY_MODE_RAIN) {
		if (RainInfoFound) { // only if data was found
			displayTemp.drawLine(0,0,0,63);     // y-axis
			displayTemp.drawLine(0,63,127,63);  // x-axis
			displayTemp.setFont(ArialMT_Plain_16);
			displayTemp.setTextAlignment(TEXT_ALIGN_RIGHT);    
			displayTemp.drawString(128, 38, ".1");
			displayTemp.drawLine(0,48,112,48);  // hor line at 0.1 mm/hr
			displayTemp.setTextAlignment(TEXT_ALIGN_RIGHT);    
			displayTemp.drawString(128, 22, "1");
			displayTemp.drawLine(0,32,112,32);  // hor line at 1 mm/hr
			displayTemp.setTextAlignment(TEXT_ALIGN_RIGHT);    
			displayTemp.drawString(128, 6, "10");
			displayTemp.drawLine(0,16,112,16);  // hor line at 10 mm/hr
			for (int n = 0; n < 24; n++) displayTemp.fillRect(5*n,63-Rain5minData[n],5,Rain5minData[n]);
		}
		else { // display no info text
			displayTemp.setFont(ArialMT_Plain_16);
			displayTemp.setTextAlignment(TEXT_ALIGN_LEFT);    
			displayTemp.drawString(0, 24, "NO RAIN INFO!");
		}
	}
	if (DisplayMode==DISPLAY_MODE_SOLAR) {
		if (SolarInfoFound) { // only if data was found
			displayTemp.setFont(ArialMT_Plain_16);
			displayTemp.drawLine(28,63-13,120,63-13);	// hor line at 0.5 KWH
			displayTemp.drawLine(28,63-26,100,63-26);	// hor line at 1 KWH
			displayTemp.drawLine(28,63-39,100,63-39);	// hor line at 1.5 KWH
			displayTemp.drawLine(0,63-53,100,63-53);	// hor line at 2.0 KWH
			displayTemp.setTextAlignment(TEXT_ALIGN_LEFT);    
			displayTemp.drawString(0, 63-14-7, "0.5");
			displayTemp.drawString(0, 63-28-7, "1.0");
			displayTemp.drawString(0, 63-42-7, "1.5");
			char cstr[5];
			float s;
			float f;
			for (int n = 0; n < 24; n++) {
				s += SolarHourData[n];
				displayTemp.fillRect(5*n, 63-(SolarHourData[n]/38), 5, (SolarHourData[n]/38) );
				if (CurrentHour.toInt()==n) { 
					f = SolarHourData[n]; // show in KWH
					dtostrf(f/1000,4, 1, cstr); // only 1 decimal digit
					displayTemp.setTextAlignment(TEXT_ALIGN_RIGHT);    
					displayTemp.drawString(128, 20, cstr);
				}
			}
			dtostrf(s/1000,4, 1, cstr); // show in KWH, only 1 decimal digit e.g. 12.3
			displayTemp.setTextAlignment(TEXT_ALIGN_RIGHT);    
			displayTemp.drawString(128, 0, cstr);
		}
		else { // display no info text
			displayTemp.setFont(ArialMT_Plain_16);
			displayTemp.setTextAlignment(TEXT_ALIGN_LEFT);    
			displayTemp.drawString(0, 24, "NO SOLAR INFO!");
		}
	}
	// now actually show the buffer data on the TEMP display
	displayTemp.display();

	// set the ALARM LED color to RED, ORANGE, YELLOW or GREEN:
	// AlarmCode = InfoFunction; // Used for testing the RGB LED. It toggles led between green-yellow-orange-red.
	if (AlarmCode==0) { // GREEN lowest code (100% GREEN + 0% RED)
		digitalWrite(LedRed, LOW);
		digitalWrite(LedGreen1, HIGH);
		digitalWrite(LedGreen2, HIGH);
	}
	if (AlarmCode==1) { // YELLOW (100% GREEN + 100% RED)
		digitalWrite(LedRed, HIGH);
		digitalWrite(LedGreen1, HIGH);
		digitalWrite(LedGreen2, HIGH);
	}
	if (AlarmCode==2) { // ORANGE (50% GREEN + 100% RED)
		digitalWrite(LedRed, HIGH);
		digitalWrite(LedGreen1, HIGH);
		digitalWrite(LedGreen2, LOW);
	}
	if (AlarmCode==3) { // RED highest code (0% GREEN + 100% RED)
		digitalWrite(LedRed, HIGH); 
		digitalWrite(LedGreen1, LOW);
		digitalWrite(LedGreen2, LOW);
	}
	InfoFunction++;
	if (InfoFunction >= 9) InfoFunction = 0;
	// Serial.println("InfoFunction: "+String(InfoFunction));

	FetchData++;
	if (FetchData >= IntervalDataFetch) FetchData = 0;

	// these lines are included during development to detect memory usage;
	CurrHeap = ESP.getFreeHeap();
	if (CurrHeap-PrevHeap!=0) {
		DPRINTLN("Free memory: "+ String(CurrHeap) + ", change: " + String(CurrHeap - PrevHeap)+" Bytes.");
		DPRINTLN("HeapFragmentation: "+ String(ESP.getHeapFragmentation()) + "%.");
	}
	PrevHeap = CurrHeap;

	delay(LoopTime);
}
