// *********************
// *** WIFI connect  ***
// *********************
void connectLocalWiFi() {
	// Connect to local WiFi-netwerk
	ConnectsCount++;
	Serial.println();
	Serial.print("Connect to: ");
	Serial.println(PRIVATE_WIFI_SSID);

	WiFi.begin(PRIVATE_WIFI_SSID, PRIVATE_WIFI_PASSWORD);

	while (WiFi.status() != WL_CONNECTED) {
		delay(500);
		Serial.print(".");
	}
	Serial.println("");
	Serial.println("WiFi connected!");
	Serial.print("IP-adres: ");	Serial.println(WiFi.localIP());
	Serial.print("Signal strength: ");Serial.println(WiFi.RSSI());
	Serial.println("");
}
// *********************
// ***  TIME SERVER  ***
// *********************
void getLocalDateTime() {

	String inputLine = "";
	bool dataFound = 0;
	int sepPos1 = 0;
	int sepPos2 = 0;
	int lenTot =0;
	
	// Use WiFiClient class to create TCP connections to TIME SERVER
	DPRINT("Setting up TCP connection with TIME SERVER: ");DPRINTLN(hostT);
	clientS.setInsecure();
	if (!clientS.connect(hostT, httpsPort)) {
		Serial.println("Connection failed!!!");
	}
	else {
		DPRINT("Request url: ");DPRINTLN(PRIVATE_URLT);
		clientS.println("GET " + String(PRIVATE_URLT) + " HTTP/1.1");
		clientS.println("Host: " + String(hostT));
		clientS.println("Connection: close");
		clientS.println(); // this signals end of header to server
		
		// give the server some time to respond
		unsigned long timeout = millis() + 5000;
		while (clientS.available() == 0) {
			if (timeout - millis() < 0) {
				Serial.println("Client Timeout!!!");
				clientS.stop();
			}
		}
	}
	// process the time server response
	// Find the line with the DATE info, format "yyyy-mm-dd"
	dataFound = 0;
	while (clientS.available()&& dataFound == 0) {
		inputLine = clientS.readStringUntil('\r\n');
		//DPRINTLN("Date header inputLine: " + inputLine);            
		if (inputLine.indexOf("yyyy-mm-dd") != -1) { 
			dataFound = 1;
			sepPos1 = inputLine.indexOf("=");
			sepPos2 = inputLine.indexOf("<br />");
			CurrentDate = inputLine.substring(sepPos1+1,sepPos2);
			DPRINTLN("CurrentDate: " + CurrentDate);
		}
	}
	// Find the line with the TIME info, format "hh:mm" 24 hour notation, corrected for DST
	dataFound = 0;
	while (clientS.available()&& dataFound == 0) {
		inputLine = clientS.readStringUntil('\r\n');
		//DPRINTLN("Time header inputLine: " + inputLine);            
		if (inputLine.indexOf("hh:mm") != -1) { 
			dataFound = 1;
			sepPos1 = inputLine.indexOf("=");
			sepPos2 = inputLine.indexOf("<br />");
			CurrentTime = inputLine.substring(sepPos1+1,sepPos2);
			DPRINTLN("CurrentTime: " + CurrentTime);
			CurrentHour = CurrentTime.substring(0,2);
			DPRINTLN("CurrentHour: " + CurrentHour);
		}
	}
	// Empty the html input buffer
	while (clientS.available()) int x = clientS.read();
}
// ***********************
// *** WEATHER SECTION *** 
// ***********************
bool getWeatherInfo() {

		bool dataFound = 0;
		String inputLine = "";
		int commaPos = 0;
		int colonPos = 0;
		int lenTot = 0;
		String tagName;
		String tagValue;

	// Use WiFiClient class to create TCP connections to WEATHER HOST
	DPRINT("Setting up TCP connection with WEATHER SERVER: ");DPRINTLN(hostW);
	if (!clientH.connect(hostW, httpPort)) {
		Serial.println("Connection failed!!!");
	}
	else { 
		// fetch the weather conditions info
		DPRINTLN("Request url: "+urlW);
		clientH.println("GET " + urlW + " HTTP/1.1");
		clientH.println("Host: " + String(hostW));
		clientH.println("Connection: close");
		clientH.println(); // this signals end of header to server
		
		// give the server some time to respond
		unsigned long timeout = millis() + 5000;
		while (clientH.available() == 0) {
			if (timeout - millis() < 0) {
				Serial.println("Client Timeout!!!");
				clientH.stop();
			}
		}
	}
	// process the WEATHER info data in  the HTML buffer
	// Find the line with the weather information
	dataFound = 0;
	while (clientH.available()&& dataFound == 0) {  
		inputLine = clientH.readStringUntil('\r\n');
		DPRINTLN("Weather header inputLine: " + inputLine);            
		if (inputLine.indexOf("liveweer") != -1) dataFound = 1;
	}
	// Empty the html input buffer
	while (clientH.available()) int x = clientH.read();

	if (!dataFound) {
		return 0;
	}
	else {
		// strip the prefix from the inputLine string
		commaPos = inputLine.indexOf("[{")+2;
		lenTot = inputLine.length();
		inputLine = inputLine.substring(commaPos, lenTot);
		
		// At this point InputLine only holds all the weatherinfo
		// Tags + values each separated by ",". Tag and value separated by ":"
		// Start reading the weather info one item at a time.
		do  {
		tagName = inputLine.substring(1,colonPos-1);
		commaPos = inputLine.indexOf(",");
		if (commaPos == -1) commaPos = inputLine.indexOf("}");
		tagValue = inputLine.substring(colonPos+3,commaPos-1);
		if (tagName=="plaats") {         // de locatie, eg Amsterdam
		}
		if (tagName=="temp") {           // actuele temperatuur in graden celcius
			TemperatureAir = tagValue;
			DPRINTLN("TemperatureAir: " + TemperatureAir);
			// reset min temp value at 16:00 (max temp) of the day
			if ((PreviousHour=="15") && (CurrentHour=="16")) TempLow = TemperatureAir;
			// reset max temp value at 06:00 (min temp) of the day
			if ((PreviousHour=="05") && (CurrentHour=="06")) TempHigh = TemperatureAir;
			if ((TemperatureAir.toFloat()) < (TempLow.toFloat())) TempLow = TemperatureAir;
			if ((TemperatureAir.toFloat()) > (TempHigh.toFloat())) TempHigh = TemperatureAir;
		}
		if (tagName=="gtemp") {          // gevoelstemperatuur in graden celcius
			TemperatureFeel = tagValue;
			DPRINTLN("TemperatureFeel: " + TemperatureFeel);
		}
		if (tagName=="samenv") {         // omschrijving weersgesteldheid als text in paar woorden eg 'half bewolkt'
			WeatherInfo = tagValue;
			DPRINTLN("WeatherInfo: " + WeatherInfo);
		}
		if (tagName=="lv") {             // relatieve luchtvochtigheid in %
			Humidity = tagValue;
			DPRINTLN("Humidity: " + Humidity);
		}
		if (tagName=="windr") {          // windrichting eg N, O, NO, ZZW
			WindDirection = tagValue;
			DPRINTLN("WindDirection: " + WindDirection);
		}
		if (tagName=="windms") {         // windsnelheid in meter per seconde
		}
		if (tagName=="winds") {          // windkracht (Beaufort)
			WindForce = tagValue;
			DPRINTLN("WindForce: " + WindForce);
		}
		if (tagName=="windk") {          // Windsnelheid in knopen
		}
		if (tagName=="windkmh") {        // Windsnelheid in km/h, eg 28.8
		}
		if (tagName=="luchtd") {         // luchtdruk in mbar eg 1018.9
			AirPressure = tagValue;
			// next 2 lines removes the fraction part
			AirPressureVal = AirPressure.toInt();
			AirPressure = String(AirPressureVal); 
			if (AirPressurePrev==0) AirPressurePrev = AirPressureVal;
			// set the pressure trend. check difference every hour
			if (PreviousHour!=CurrentHour) {
				AirPressureTrend = " ";
				if ((AirPressureVal-AirPressurePrev)>=1) AirPressureTrend = "+";
				if ((AirPressureVal-AirPressurePrev)>=2) AirPressureTrend = "++";
				if ((AirPressureVal-AirPressurePrev)>=5) AirPressureTrend = "+++";
				if ((AirPressureVal-AirPressurePrev)<=-1) AirPressureTrend = "-";
				if ((AirPressureVal-AirPressurePrev)<=-2) AirPressureTrend = "--";
				if ((AirPressureVal-AirPressurePrev)<=-5) AirPressureTrend = "---";
				DPRINTLN("AirPressureTrend: " + AirPressureTrend);
				AirPressurePrev = AirPressureVal;
				PreviousHour = CurrentHour;
			}
			DPRINTLN("AirPressure: " + AirPressure);
		}
		if (tagName=="ldmmhg") {         // luchtdruk in mm-kwikdruk eg 764
		}
		if (tagName=="dauwp") {          // dauwpunt in graden celcius
		}
		if (tagName=="zicht") {          // zicht in km eg 35
			Visibility = tagValue;
			DPRINTLN("Visibility: " + Visibility);
		}
		if (tagName=="verw") {           // dagverwachting text als zin
		}
		if (tagName=="sup") {            // tijd zon op in hh:mm 
			SunRiseTime = tagValue;
			DPRINTLN("SunRiseTime: " + SunRiseTime);
		}
		if (tagName=="sunder") {         // tijd zon onder in hh:mm
			SunSetTime = tagValue;
			DPRINTLN("SunSetTime: " + SunSetTime);
		}
		if (tagName=="image") {          // één-woord-weer, eg zonnig
		}
		if (tagName=="d0weer") {         // icoon afbeeldingsnaam vandaag eg 'sneeuw'
		}
		if (tagName=="d0tmax") {         // Maxtemp vandaag
			DagTempMax = tagValue;
		}
		if (tagName=="d0tmin") {         // Mintemp vandaag
			DagTempMin = tagValue;
		}
		if (tagName=="d0windk") {        // Windkracht vandaag (in Bft)
		}
		if (tagName=="d0windr") {        // Windrichting vandaag
		}
		if (tagName=="d0neerslag") {     // Neerslagkans vandaag (%)
			RainChance = tagValue;
			DPRINTLN("RainChance: " + RainChance);
		}
		if (tagName=="d0zon") {          // Zonkans vandaag (%)
			SunChance = tagValue;
			DPRINTLN("SunChance: " + SunChance);
		}
		if (tagName=="d1weer") {         // Weericoon morgen
		}
		if (tagName=="d1tmax") {         // Maxtemp morgen
		}
		if (tagName=="d1tmin") {         // Mintemp morgen
		}
		if (tagName=="d1windk") {        // Windkracht morgen (in Bft)
		}
		if (tagName=="d1windr") {        // Windrichting morgen
		}
		if (tagName=="d1neerslag") {     // Neerslagkans morgen (%)
		}
		if (tagName=="d1zon") {          // Zonkans morgen (%)
		}
		if (tagName=="d2tmax") {         // Maxtemp overmorgen 
		}
		if (tagName=="d2tmin") {         // Mintemp overmorgen
		}
		if (tagName=="d2weer") {         // Weericoon overmorgen 
		}
		if (tagName=="d2windk") {        // Windkracht overmorgen (in Bft) 
		}
		if (tagName=="d2windr") {        // Windrichting overmorgen 
		}
		if (tagName=="d2neerslag") {     // Neerslagkans overmorgen (%) 
		}
		if (tagName=="d2zon") {          // Zonkans overmorgen (%)
		}
		if (tagName=="alarm") {          // Geldt er een weerwaarschuwing voor deze regio of niet?
			if (tagValue=="3") { AlarmText = "ROOD!!"; AlarmCode = 3;}
			if (tagValue=="2") { AlarmText = "ORANJE"; AlarmCode = 2;}
			if (tagValue=="1") { AlarmText = "Geel"; AlarmCode = 1;}
			if (tagValue=="0") { AlarmText = "groen"; AlarmCode = 0;}
			DPRINTLN("AlarmText: " + AlarmText);
		}
		if (tagName=="alarmtxt") // Only present when 'alarm' is <> "0".
		
		// shorten the weatherinfo line with handled tag+value to continue with the next tag+value
		lenTot = inputLine.length();
		inputLine = inputLine.substring(commaPos+2,lenTot);
		colonPos = inputLine.indexOf(":");
		} while (colonPos != -1);
	}
	return 1;
}
// ********************
// *** RAIN SECTION ***
// ********************
bool getRainInfo() {

		bool dataFound = 0;
		String inputLine = "";

	// Use WiFiClient class to create TCP connections to RAIN hostR
	clientS.setInsecure();
	DPRINT("Setting up TCP connection with RAIN SERVER: ");DPRINTLN(hostR);
	if (!clientS.connect(hostR, httpsPort)) {
		Serial.println("Connection failed!!!");
	}
	else { // connection was succes
		DPRINTLN("Request urlR: "+urlR);
		clientS.println("GET " + urlR + " HTTP/1.1");
		clientS.println("Host: " + String(hostR));
		clientS.println("Connection: close");
		clientS.println(); // this signals end of header to server

		// give the server some time to respond
		unsigned long timeout = millis() + 5000;
		while (clientS.available() == 0) {
			if (timeout - millis() < 0) {
				Serial.println("Client Timeout!!!");
				clientS.stop();
			}
		}
	}
	// process the RAIN info data from the HTML buffer
	while (clientS.available()&& dataFound == 0) {
		inputLine = clientS.readStringUntil('\r\n');
		//DPRINTLN("Rain header inputLine: " + inputLine);            
		if (inputLine.indexOf("HTTP/1.1 200 OK") != -1) dataFound = 1;
	}
	if (dataFound) {
		unsigned int n = 0;
		while (clientS.available()) {
			inputLine = clientS.readStringUntil('\r\n');
			if (inputLine.charAt(3)=='|') {
				Rain5minData[n] = inputLine.substring(0,3).toInt();
				// convert range of x=000...255 to d=0-64;
				// rain mm/hr = 10^((x-109)/32), x=77 gives 0.1mm/hr, x=109 gives 1 mm/hr, x=141 gives 10 mm/hr
				// use formula d = x/2-27 for display. This gives a display range of 0.01 mm/hr (d=0) to 100 mm/hr (d=63)
				// and practical display values of d=16 is 0.1 mm/hr, d=32 is 1 mm/hr, d=48 is 10 mm/hr
				// the translation from d -> mm/hr becomes: mm/hr = 10^((d-32)/16
				// Rain5minData[n] = random(163); // for testing purposes only
				Rain5minData[n] = Rain5minData[n] / 2 - 27;
				if (Rain5minData[n]<0) Rain5minData[n]=0;
				if (Rain5minData[n]>63) Rain5minData[n]=63;
				DPRINT("Rain data InputLine: " + inputLine);
				DPRINTLN(" - Value: " + String(Rain5minData[n]));
				n++;
			}
		}
	} 
	// Empty the html input buffer
	while (clientS.available()) int x = clientS.read();
	return 1;
}
// *********************
// *** SOLAR SECTION ***
// *********************
bool getSolarInfo() {

	const String urlS = "/site/" + String(PRIVATE_SOLAR_SITE) + "/energy?timeUnit=HOUR&endDate=" + CurrentDate + "&startDate=" + CurrentDate + "&api_key=" + String(PRIVATE_SOLAR_APIKEY);
	bool dataFound = 0;
	String inputLine = "";
	String tagValue = "";
	int lenTot = 0;
	int sepPos1 = 0;
	int sepPos2 = 0;

	// Use WiFiClient class to create TCP connections to Solar hostS
	DPRINT("Setting up TCP connection with SOLAR SERVER: ");DPRINTLN(hostS);
	clientS.setInsecure();
	if (!clientS.connect(hostS, httpsPort)) {
		Serial.println("Connection failed!!!");
		return 0;
	}
	else { // connection was succes
		DPRINTLN("Request urlS: "+urlS);
		clientS.println("GET " + urlS + " HTTP/1.1");
		clientS.println("Host: " + String(hostS));
		clientS.println("Connection: close");
		clientS.println(); // this signals end of header to server

		// give the server some time to respond
		unsigned long timeout = millis() + 5000;
		while (clientS.available() == 0) {
			if (timeout - millis() < 0) {
				Serial.println("Client Timeout!!!");
				clientS.stop();
				return 0;
			}
		}
	}
	// process the SOLAR info data from the HTML buffer
	while (clientS.available()&& dataFound == 0) {
		inputLine = clientS.readStringUntil('\r\n');
		//DPRINTLN("Solar header inputLine: " + inputLine);            
		if (inputLine.indexOf("energy") != -1) dataFound = 1;
	}
	if (dataFound) {
		// inputLine = {"energy":{"timeUnit":"HOUR","unit":"Wh","measuredBy":"INVERTER","values":[{"date":"2020-05-14 00:00:00","value":null},,,,{"date":"2020-05-14 23:00:00","value":null}]}}
		sepPos1 = inputLine.indexOf("[{")+2;
		lenTot = inputLine.length();
		inputLine = inputLine.substring(sepPos1, lenTot);
		//DPRINTLN("Solar data inputLine: " + inputLine);
		// inputLine = "date":"2020-05-14 00:00:00","value":null},,,,{"date":"2020-05-14 23:00:00","value":null}]}}		
		for (int n = 0; n < 24; n++) {
			sepPos1 = inputLine.indexOf("value");
			sepPos2 = inputLine.indexOf("}");
			tagValue = inputLine.substring(sepPos1+7,sepPos2);
			SolarHourData[n] = 0;
			if (tagValue!="null") SolarHourData[n] = tagValue.toInt();
			DPRINT("Solar value hour: " + String(n) + " : ");
			DPRINTLN(tagValue + " -> " + String(SolarHourData[n]));
			// advance to next hour ...
			lenTot = inputLine.length();
			inputLine = inputLine.substring(sepPos2+1,lenTot);
		}
	} 
	// Empty the html input buffer
	while (clientS.available()) int x = clientS.read();
	return 1;
}
